@@ -38,10 +38,8 @@ my $title_image;
 
 # should go to utility module
 sub dir_is_movie($) {
-   return undef; #d# not ready yet
-
    -f "$_[0]/VIDEO_TS/VIDEO_TS.IFO"		and return "dvd";
-   -d "$_[0]/BDMV/STREAM"			and return "br";
+   -d "$_[0]/BDMV/STREAM/."			and return "br";
 
    undef
 }
@@ -176,7 +174,7 @@ sub set_image {
    $self->{type}        = $type;
    $self->{image}       = $image;
    $self->{path}        = undef;
-   $self->{tran_rotate} = $self->{load_rotate} %= 360;
+   $self->{tran_rotate} = $self->{load_rotate};
 
    $image =  Gtk2::CV::rotate $image, -$self->{load_rotate}
       if $self->{load_rotate};
@@ -290,21 +288,22 @@ my %exttypes = (
 sub load_image {
    my ($self, $path) = @_;
 
+   my ($image, $type);
+
    eval {#d#
       $self->kill_player;
       $self->force_redraw;
 
       my $moviedir = dir_is_movie $path;
 
-      my $image;
-      my $type;
-
       # magic/file is no longer very reliable
       if (open my $fh, "<:raw", $path) {
-         if (sysread $fh, my $buf, 64) {
+         if (sysread $fh, my $buf, 1024) { # 64 for everybody, except gif, which needs ~1kb
             unless (defined ($type = Gtk2::CV::filetype $buf)) {
                if ($buf =~ /^....ftypavif/) {
                   $type = "image/avif";
+               } elsif ($buf =~ /^\x00+\z/) { # possibly iso9660 or udf
+                  $type = "video/iso-bluray"; #d# just more commno for me nowadays
                }
                #elsif ($buf =~ /^\xff\xd8\xff\xe0\x00.JFIF\x00\x01[\x00\x01\x02]/) { $type = "image/jpeg"; # not the only possible format
             }
@@ -340,11 +339,6 @@ sub load_image {
 
       $@ = "generic file display error";
 
-      if ($type eq "image/gif") {
-         # could be animation, very hard to detect, for the time being, force mpv
-         $type = "video/gif";
-      }
-
       if (!$type or $type =~ /^text\//) {
          $@ = "unrecognised file format";
          # should, but can't
@@ -368,7 +362,7 @@ sub load_image {
 
       } elsif ($type eq "application/pdf") {
          # hack, sorry, unsupported, should use mimetools etc.
-         system "xpdf \Q$path\E &";
+         system "mupdf \Q$path\E &";
 
       } elsif ($type =~ /^image\//) {
          open my $fh, "<", $path
@@ -428,11 +422,14 @@ sub load_image {
 
          my ($w, $h);
 
-         if (1) {
+         {
             no integer;
 
+            my $ffmpeg_protocol = $type eq "video/iso-bluray" ? "bluray" : "file";
+
             # use ffmpeg's ffprobe to detect the video
-            my $ffmpeg = qx{ffprobe -v quiet -show_streams -of json -- \Q$path};
+            # TODO: should just load it into mpv and use that
+            my $ffmpeg = qx{ffprobe -v quiet -show_streams -of json -- $ffmpeg_protocol:\Q$path};
 
             # ffmpeg doesn't properly encode json
             $ffmpeg = eval { JSON::XS->new->latin1->decode ($ffmpeg) };
@@ -442,6 +439,13 @@ sub load_image {
                   $w = $_->{width};
                   $h = $_->{height};
 
+                  # apply display matrix rotation side data
+                  if (my ($dm) = grep $_->{side_data_type} eq "Display Matrix", @{ $_->{side_data_list} }) {
+                     if ((int $dm->{rotation} / 90) & 1) {
+                        ($w, $h) = ($h, $w);
+                     }
+                  }
+
                   if ($_->{sample_aspect_ratio} =~ /^(\d+):(\d+)$/ && $1 && $2) {
                      $w = POSIX::ceil $w * ($1 / $2);
                   }
@@ -454,29 +458,10 @@ sub load_image {
                }
             }
 
-            # hack, ffmpeg does not support dvds
+            # hack/TODO, ffmpeg does not support dvds
             if ($moviedir && !$w) {
                ($w, $h) = (1280, 720);
             }
-
-
-         } else {
-            no integer;
-
-            die "no longer works with mpv";
-
-            my $mpv = qx{LC_ALL=C exec \Q$Gtk2::CV::MPV\E </dev/null 2>/dev/null -really-quiet -noconfig all -noautosub -nofontconfig -sub /dev/null -sub-fuzziness 0 -cache-min 0 -input nodefault-bindings:conf=/dev/null -identify -vo null -ao null -frames 0 \Q$path};
-
-            $w = $mpv =~ /^ID_VIDEO_WIDTH=(\d+)$/sm ? $1 : undef;
-            $h = $mpv =~ /^ID_VIDEO_HEIGHT=(\d+)$/sm ? $1 : undef;
-
-            if ($w && $mpv =~ /^ID_VIDEO_ASPECT=([0-9\.]+)$/sm && $1 > 0) {
-               $w = POSIX::ceil $w * $1 * ($h / $w); # correct aspect ratio, assume square pixels
-            } else {
-               # no idea what to do, mpv's aspect factors seem to be random
-               #$w = POSIX::ceil $w * 1.50 * ($h / $w); # correct aspect ratio, assume square pixels
-               #$w = POSIX::ceil $w * 1.33;
-            }
          }
 
          my @extra_mpv_args;
@@ -496,7 +481,6 @@ sub load_image {
          }
 
          if ($w && $h) {
-            $type = "video";
             $image = new Gtk2::Gdk::Pixbuf "rgb", 0, 8, $w, $h;
             $image->fill ("\0\0\0");
 
@@ -530,43 +514,69 @@ sub load_image {
             $self->{player_pid} = fork;
 
             if ($self->{player_pid} == 0) {
-               $ENV{LC_ALL} = "C";
+               $ENV{LC_ALL} = "POSIX";
 
                my @rotate = (
                   [],
-                  ["-vf-add", "rotate=0"],
-                  ["-vf-add", "flip", "-vf-add", "mirror"],
-                  ["-vf-add", "rotate=2"],
+                  ["--video-rotate=90"],
+                  ["--video-rotate=180"],
+                  ["--video-rotate=270"],
+                  # mplayer needs this:
+                  #[],
+                  #["-vf-add", "rotate=0"],
+                  #["-vf-add", "flip", "-vf-add", "mirror"],
+                  #["-vf-add", "rotate=2"],
                );
 
                my @mpv_args;
 
                @mpv_args = (
                   qw(
-                     --no-terminal --no-input-terminal --no-input-default-bindings --no-input-cursor --input-conf=/dev/null
-                     --input-vo-keyboard=no -loop-file=inf
+                     --really-quiet
+                     --no-terminal --no-input-terminal --no-input-default-bindings
+                     --no-input-cursor --input-conf=/dev/null
+                     --sub-auto=all
+                     --autoload-files=yes
+                     --input-vo-keyboard=no --loop-file=inf
                   ),
-                  @extra_mpv_args,
+
+                  @{ $rotate[$self->{load_rotate} / 90] },
                );
 
+               if ($type eq "video/gif") {
+                  push @mpv_args, "--autoload-files=no";
+               }
+
+               push @mpv_args, @extra_mpv_args;
+
                $Gtk2::CV::mpv_list_options //= qx<$Gtk2::CV::MPV --list-options>;
 
-               if ($Gtk2::CV::mpv_list_options =~ /^--input-ipc-file\s/m) {
-                  push @mpv_args, "--input-ipc-file=fd://" . fileno $sfh;
+               if ($Gtk2::CV::mpv_list_options =~ /\s--input-ipc-client\s/m) {
+                  push @mpv_args, "--input-ipc-client=fd://" . fileno $sfh;
                } else {
-                  # we always assume --input-file, as list-options doens't list it even if it is there
+                  # we always assume --input-file, as list-options doesn't list it even if it is there
                   push @mpv_args, "--input-file=fd://" . fileno $sfh;
                }
 
+               #open STDERR, ">/dev/null";
+
+               my @movie_args = ("--", $path);
+
+               if ($moviedir) {
+                  if ($moviedir eq "br") {
+                     @movie_args = ("--bluray-device=$path", "bd://");
+                  } elsif ($moviedir eq "dvd") {
+                     @movie_args = ("--dvd-device=$path", "dvd://");
+                  }
+               } elsif ($type eq "video/iso-bluray") {
+                  @movie_args = ("--bluray-device=$path", "bd://");
+               }
+
                open STDIN, "</dev/null";
                open STDOUT, ">/dev/null";
                fcntl $sfh, Fcntl::F_SETFD (), 0;
 
-               #open STDERR, ">/dev/null";
-
-               exec $Gtk2::CV::MPV, @mpv_args, qw(-quiet),
-                               @{ $rotate[$self->{load_rotate} / 90] },
-                               -wid => $xid, $path;
+               exec $Gtk2::CV::MPV, @mpv_args, -wid => $xid, @movie_args;
                POSIX::_exit 0;
             }
 
@@ -605,23 +615,23 @@ sub load_image {
       } else {
          $@ = "unrecognized file format '$type'";
       }
-
-      if (!$image) {
-         warn "$@";
-
-         $type = "error";
-         $image = Gtk2::CV::require_image "error.png";
-      }
-
-      if ($image) {
-         $self->set_image ($image, $type);
-         $self->{path} = $path;
-         $self->set_title ("CV: $path");
-      } else {
-         $self->clear_image;
-      }
    };#d#
    warn "$@" if $@;#d#
+
+   if (!$image) {
+      warn "$@";
+
+      $type = "error";
+      $image = Gtk2::CV::require_image "error.png";
+   }
+
+   if ($image) {
+      $self->set_image ($image, $type);
+      $self->{path} = $path;
+      $self->set_title ("CV: $path");
+   } else {
+      $self->clear_image;
+   }
 }
 
 sub reload {
@@ -999,12 +1009,16 @@ sub handle_key {
       } elsif ($key == $Gtk2::Gdk::Keysyms{t}) {
          my $r = $::REVERSE_ROTATION ? +90 : -90;
          $self->set_subimage (Gtk2::CV::rotate $self->{subimage}, $r);
-         $self->{tran_rotate} -= $r;
+         ($self->{tran_rotate} += 360 - $r) %= 360;
+         printf {$self->{mpv_fh}} "set video-rotate %d\n", $self->{tran_rotate}
+            if $self->{mpv_fh};
 
       } elsif ($key == $Gtk2::Gdk::Keysyms{T}) {
          my $r = $::REVERSE_ROTATION ? -90 : +90;
          $self->set_subimage (Gtk2::CV::rotate $self->{subimage}, $r);
-         $self->{tran_rotate} -= $r;
+         ($self->{tran_rotate} += 360 - $r) %= 360;
+         printf {$self->{mpv_fh}} "set video-rotate %d\n", $self->{tran_rotate}
+            if $self->{mpv_fh};
 
       } elsif ($key == $Gtk2::Gdk::Keysyms{a}) {
          $self->{path}
@@ -1013,7 +1027,7 @@ sub handle_key {
          $self->{type} eq "image/jpeg"
             or die "image has type '$self->{type}', but I can only 'a'pply jpeg images";
          
-         my $rot = $self->{tran_rotate} %= 360;
+         my $rot = $self->{tran_rotate};
 
          $rot = $rot ==   0 ? undef
               : $rot ==  90 ? -9
@@ -1046,10 +1060,10 @@ sub handle_key {
          print {$self->{mpv_fh}} "seek +600\n";
       } elsif ($self->{player_pid} && $key == $Gtk2::Gdk::Keysyms{Page_Down}) {
          print {$self->{mpv_fh}} "seek -600\n";
-      } elsif ($self->{player_pid} && $key == $Gtk2::Gdk::Keysyms{'#'}) {
-         print {$self->{mpv_fh}} "switch_audio\n";
+      } elsif ($self->{player_pid} && $key == $Gtk2::Gdk::Keysyms{numbersign}) {
+         print {$self->{mpv_fh}} "cycle audio\n";
       } elsif ($self->{player_pid} && $key == $Gtk2::Gdk::Keysyms{j}) {
-         print {$self->{mpv_fh}} "sub_select\n";
+         print {$self->{mpv_fh}} "cycle sub\n";
       } elsif ($self->{player_pid} && $key == $Gtk2::Gdk::Keysyms{o}) {
          print {$self->{mpv_fh}} "no-osd cycle-values osd-level 2 3 0 1\n";
       } elsif ($self->{player_pid} && $key == $Gtk2::Gdk::Keysyms{p}) {
