@@ -45,7 +45,7 @@ sub INIT_INSTANCE {
 
    $self->{list}->signal_connect (cursor_changed => sub {
       my @rows = $_[0]->get_selection->get_selected_rows
-         or return;
+         or return 1;
 
       my $row = scalar $rows[0]->get_indices;
 
@@ -112,7 +112,7 @@ sub analyse {
                          s/^.*\///;
                          s/(?:-\d+|\.~[~0-9]*)+$//; # remove -000, .~1~ etc.
                          s/\.[^.]*$//;
-                         ($orig => [/(\pL(?:\pL+|\pP(?=\pL))* | \pN+)/gx])
+                         ($orig => [/(\pL(?:\pL+)* | \pN+)/gx])
                       }
                       grep !/\.(sfv|crc|par|par2)$/i,
                            @{ $self->{select}[-1] };
@@ -228,8 +228,8 @@ sub new_schnauzer {
             my ($path) = $namecluster->{list}->get_cursor;
             $key == $Gtk2::Gdk::Keysyms{Up} ? $path->prev : $path->next;
             $namecluster->{list}->set_cursor ($path);
-            return 1;
          }
+         return 1;
       }
 
       0
