@@ -354,11 +354,11 @@ sub {
       die "can only rotate regular files"
          unless Fcntl::S_ISREG ($job->{stat}[2]);
 
-      $rot = $rot eq "auto"      ? "-a"
-           : ($rot % 360) ==   0 ? undef
-           : ($rot % 360) ==  90 ? -9
-           : ($rot % 360) == 180 ? -1
-           : ($rot % 360) == 270 ? -2
+      $rot = $rot eq "auto" ? "-a"
+           : $rot ==   0    ? undef
+           : $rot ==  90    ? -9
+           : $rot == 180    ? -1
+           : $rot == 270    ? -2
            : die "can only rotate by 0, 90, 180 and 270 degrees";
 
       if ($rot) {
@@ -390,7 +390,13 @@ sub video_thumbnail_at {
          or return;
 
       #system "exec mpv -really-quiet -noautosub -nosound -input nodefault-bindings -noconfig all -ss $time -frames 2 -vf scale=" . IWG . ":-2::::::1 -vo pnm:outdir=\Q$dir\E \Q$path\E >/dev/null 2>&1 </dev/null";
-      system "exec mpv --no-terminal --sid=no --ao=null --no-config --start=$time --frames=2 --vf=scale=" . IWG . ":-2::::::1 --vo=image -vo-image-format=png --vo-image-png-compression=0 -vo-image-outdir=\Q$dir\E \Q$path\E >/dev/null 2>&1 </dev/null";
+      system "exec mpv"
+           . " --really-quiet"
+           . " --no-terminal --sid=no --ao=null --no-config"
+           . " --access-references=no --autoload-files=no"
+           . " --start=$time --frames=2 --vf=scale=" . IWG . ":-2::::::1"
+           . " --vo=image -vo-image-format=png --vo-image-png-compression=0 -vo-image-outdir=\Q$dir\E \Q$path\E"
+           . " >/dev/null 2>&1 </dev/null";
 
       my $pb = eval { new_from_file Gtk2::Gdk::Pixbuf "$dir/00000002.png" };
 
@@ -814,6 +820,23 @@ sub coord {
    );
 }
 
+# newer gtk2 versions (maybe caused by xft bugs?) change the label width
+# rather erratically even with the same font, which, when size hints are
+# used, will resize the window when the label text changes. this tries to
+# work around this, by allocating more vertical pixels for rounding
+# issues and never reducing the height, ever.
+sub fix_label_jumping {
+   my ($label) = @_;
+
+   my $max;
+
+   $label->signal_connect_after (size_request => sub {
+      my $h = $_[1]->height;
+      $max = $h + 2 if $max < $h;
+      $_[1]->height ($max);
+   });
+}
+
 sub INIT_INSTANCE {
    my ($self) = @_;
 
@@ -830,8 +853,9 @@ sub INIT_INSTANCE {
    $self->pack_start (new Gtk2::HSeparator, 0, 0, 0);
    $self->pack_end   (my $labelwindow = new Gtk2::EventBox, 0, 1, 0);
    $labelwindow->add ($self->{info} = new Gtk2::Label);
+   fix_label_jumping $self->{info};
+   # make sure the text clips to the window *sigh*
    $labelwindow->signal_connect_after (size_request => sub { $_[1]->width (0) });
-   # the above 3 lines are just to make the text clip to the window *sigh*
    $self->{info}->set (selectable => 1, xalign => 0, justify => "left");
 
    $self->signal_connect (destroy => sub { %{$_[0]} = () });
@@ -1135,7 +1159,6 @@ sub emit_sel_changed {
             $self->{info_updater_group}->feed (sub {
                my $entry = pop @$todo
                   or return;
-
                $_[0]->add (aio_stat "$entry->[E_DIR]/$entry->[E_FILE]", sub {
                   my ($size, $blocks) = (stat _)[7, 12];
                   $self->{info_size} += $size;
@@ -2157,13 +2180,14 @@ sub chpaths {
 
    my $all_group = $self->{chpaths_grp} = aio_group $cb;
 
-   $self->set_sensitive (0);
+   # get_parent_widnow is a hack hack., but otherwise gtk mishandles events
+   $self->get_toplevel->set_sensitive (0) if $self->get_toplevel;
    my $inhibit_guard = Gtk2::CV::Jobber::inhibit_guard;
 
    my $guard = Guard::guard {
       delete $self->{chpaths_grp};
       undef $inhibit_guard;
-      $self->set_sensitive (1);
+      $self->get_toplevel->set_sensitive (1) if $self->get_toplevel;
       $self->{draw}->grab_focus unless eval { $self->get_toplevel->get_focus };
    };
 
